/** @file gui++.h
 *
 * @author: marco corvi <marco.corvi@elsag.it>
 * @date  : Sept. 2002
 *
 * \brief C++ header file for the GUI class
 */
#ifndef GUI_PP_H
#define GUI_PP_H

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include <unistd.h>

#ifndef NOTHREADS
  #include <pthread.h>
  #include <signal.h>
#endif

#include <X11/Xlib.h>
#include <X11/Xos.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <X11/X.h>
#include <X11/Xatom.h>

#ifndef TRUE
#  define TRUE 1
#endif
#ifndef FALSE
#  define FALSE 0
#endif

#define BGCOLOR "0xcccccc"

#define BACK_TYPE unsigned short

#define BLACK   0x000000
#define RED     0xff0000
#define YELLOW  0xffff00
#define CYAN    0xff00ff
#define MAGENTA 0x00ffff
#define GREEN   0x00ff00
#define BLUE    0x0000ff
#define WHITE   0xffffff
#define GREY    0xcccccc

class GUI {
  private:
    Display * theDisplay;
    GC        theGC;
    int       theDepth;
    Visual  * theVisual;
    Window    W;
    XEvent  event;

    unsigned int windowWidth;   int width;
    unsigned int windowHeight;  int height;
    int       windowX;
    int       windowY;
	int       m_zoom;

    XImage  * theImage;
    XImage  * theBackImage;
    BACK_TYPE * backImage;
    int theImageDepth;

    // void    * theData;
    char    * theData;
	int     d1save;
	int     d2save;
	unsigned char * imd;

#   ifdef DO_COLOR_LOOKUP
      int       look_up[256];
#   endif

    int do_graphics; /* dafault is no-graphics; can be set by graph_init() */

#   ifndef NOTHREADS
      pthread_t tid;
#   endif

  public:
	/** Constructor.
	 * Creates a window with the specified client size.
	 * @param ig Flag: 1 to activate display, 0 to inhibit it
	 * @param w Window width
	 * @param h Window height
	 * @param zoom zoom-in factor (display is subsampled by "zoom")
	 */
    GUI( int ig=0, unsigned int w=512, unsigned int h=256, int zoom=1 );

    ~GUI();

	/* Refresh the window client area
	 */
    void refresh();
	
	void expose();

    void backdraw( char c, int * x, int * y, unsigned long col );

	/** Draw a point (1 pixel) in the client area
	 * @param x Horizontal coordinate (from left to right)
	 * @param y Vertical coordinate (from top to bottom)
	 * @param col Color as RRGGBB
	 */
    void drawPoint( int x, int y, unsigned long col=0xffffff);

	/** Draw a cross (5 pixel wide) in the client area
	 * @param x Horizontal coordinate of the center (from left to right)
	 * @param y Vertical coordinate of the center (from top to bottom)
	 * @param col Color as RRGGBB
	 */
    void drawCross( int x, int y, unsigned long col=0xff0000);


	/** Draw a solid line (1 pixel thick) in the client area
	 * @param x1 Horizontal coordinate of the starting point
	 * @param y1 Vertical coordinate of the starting point
	 * @param x2 Horizontal coordinate of the end point
	 * @param y2 Vertical coordinate of the end point
	 * @param col Color as RRGGBB
	 */
    void drawLine( int x1, int y1, int x2, int y2, unsigned long col=0xffffff);


	/** Draw a rectagular frame (solid line 1 pixel thick) in the client area
	 * @param x Horizontal coordinate of the upper left corner
	 * @param y Vertical coordinate of the upper left corner
	 * @param w Rectangle width
	 * @param h Rectangle height
	 * @param col Color as RRGGBB
	 */
    void drawRectangle( int x, int y, int w, int h, unsigned long col=0xffffff);

	/** Draw a filled rectagle in the client area
	 * @param x Horizontal coordinate of the upper left corner
	 * @param y Vertical coordinate of the upper left corner
	 * @param w Rectangle width
	 * @param h Rectangle height
	 * @param col Color as RRGGBB
	 */
    void drawBox( int x, int y, int w, int h, unsigned long col=0xffffff);

	/** Draw a string in the client area
	 * @param msg  String to be displayed
	 * @param xoff Horizontal coordinate of the left offset
	 * @param yoff Vertical coordinate of the left offset
	 * @param col Color as RRGGBB
	 */
    void displayString( const char * msg, int xoff, int yoff,
				   unsigned long col=0xffffff	);

	void drawButton( const char * msg, int xoff, int yoff, int w, int h );

	/** Draw a message string in the StatusBar area
	 * @param msg  String to be displayed
	 */
	void displayStatus( const char * fmt, ... );

    /** Display an image in the client area
	 * @param img Image pixel data
	 * @param d1 Image width
	 * @param d2 Image height
	 * @param xoff Horizontal coordinate of the upper left corner
	 * @param yoff Vertical coordinate of the upper left corner
	 * @param bpp Bytes per pixel: 4 BGRA, 3 RGB, 1 Grey
	 */
    void displayImage( unsigned char * img, int d1, int d2, 
		int xoff, int yoff,
        int bpp
	);

	/** Resize the client area
	 * @param d1 New width
	 * @param d2 New height
	 */
    void resizeWindow(unsigned int d1, unsigned int d2);

	/** Clear the client area and write the window title
	 * @param name Window title
	 */
    void clearWindow( const char * fmt, ... );

	/** Set the foreground color
	 * @param color Color as RRGGBB
	 */
	void setForeground( unsigned long color );

	/** Wait for the specifed milliseconds
	 * @param msec Milliseconds
	 */
    void waitMsecs( int msec);

	/** Wait for the specifed milliseconds or for a mouse click
	 * @param msec Milliseconds
	 * @param x Mouse click X position
	 * @param y Mouse click Y position
	 * @param msg User prompt
	 * @return Mouse button (1, 2, 3) or -1 if time expired
	 */
	int waitMsecAndMouseClick
			(int msec, int * x, int * y, const char * msg=NULL );

    /** Wait until the user press a key
	 * @param msg User prompt
	 * @return Key code
	 */
    unsigned int waitKeyPress( const char * msg=NULL );
	
	/** Wait for the specifed milliseconds or for a mouse click
	 * @param msec Milliseconds
	 * @param msg User prompt
	 * @return Key code or 0 if time expired
	 */
    unsigned int waitMsecsAndKey( int msec, const char * msg=NULL);

	/** Wait until the user clicks with the mouse
	 * @param msg User prompt
	 * @return Mouse button (1, 2, or 3)
	 */
    unsigned int waitMouseClick( const char * msg=NULL );

	/** Wait until the user clicks with the mouse
	 * @param x Mouse click X position
	 * @param y Mouse click Y position
	 * @param msg User prompt
	 * @return Mouse button (1, 2, or 3)
	 */
    unsigned int waitMouseClick(int * x, int * y, const char * msg=NULL);

	void setZoom( int z ) {
		m_zoom = ( z < 1 ) ? 1 : z;
    }
};

#endif // GUI_PP_H
